#lang scheme

(define start-state '((17 24) (24 17) (15 0) (10 0) (0 6) (0 19)))
(define test-state '((6 1) (2 13) (S V) (21 16) (21 17) (H S)))
(define test-room-state '((C V) (C S) (B S) (K V) (O S) (U V)))
(define-test-suite valid-move?-tests
  (check-true (valid-move? start-state 0 2 '((17 22) (24 17) (15 0) (10 0) (0 6) (0 19))) "Basic corridor move")
  (check-false (valid-move? start-state 0 2 '((17 21) (24 17) (15 0) (10 0) (0 6) (0 19))) "Can't reach, too long")
  (check-false (valid-move? start-state 0 2 '((17 23) (24 17) (15 0) (10 0) (0 6) (0 19))) "Can't reach, too short")
  (check-false (valid-move? test-state 1 7 '((6 1) (9 13) (S V) (21 16) (21 17) (H S))) "Can't land on +")
  (check-true (valid-move? test-state 1 2 '((6 1) (4 13) (S V) (21 16) (21 17) (H S))) "Can move through door space")
  (check-true (valid-move? start-state 1 12 '((17 24) (15 14) (15 0) (10 0) (0 6) (0 19))) "Long move")
  (check-false (valid-move? test-state 1 3 '((6 1) (2 13) (S V) (21 16) (21 17) (H S))) "Can't reach, too short #2")
  (check-true (valid-move? test-state 1 3 '((6 1) (3 13) (S V) (21 16) (21 17) (H S))) "Can land on doorway, not enter")
  (check-true (valid-move? '((5 5) (2 13) (S V) (21 16) (21 17) (H S)) 0 5 test-state) "Can move from doorway")
  (check-false (valid-move? test-state 4 3 '((6 1) (2 13) (S V) (21 16) (21 16) (H S))) "Can't land on other player")
  (check-true (valid-move? test-state 3 2 '((6 1) (2 13) (S V) (21 18) (21 17) (H S))) "Can walk through other player")
  (check-false (valid-move? '((11 9) (24 17) (15 0) (10 0) (0 6) (0 19)) 0 8 '((11 17) (24 17) (15 0) (10 0) (0 6) (0 19))) "Can't move through +s")
  (check-true (valid-move? '((1 13) (2 13) (3 13) (4 13) (5 13) (6 13)) 0 3 '((B V) (2 13) (3 13) (4 13) (5 13) (6 13))) "Big blockage, but enter room")
  (check-true (valid-move? '((1 13) (2 13) (3 13) (4 13) (5 13) (6 13)) 1 3 '((1 13) (2 13) (3 13) (4 13) (5 13) (6 13))) "All moves blocked, stay put")
  (check-true (valid-move? test-state 0 5 '((C V) (2 13) (S V) (21 16) (21 17) (H S))) "Basic move in")
  (check-true (valid-move? test-state 0 6 '((O V) (2 13) (S V) (21 16) (21 17) (H S))) "Multi door move in")
  (check-false (valid-move? test-state 0 5 '((S V) (2 13) (S V) (21 16) (21 17) (H S))) "Can't move that far")
  (check-true (valid-move? test-state 3 3 '((6 1) (2 13) (S V) (D V) (21 17) (H S))) "Multi door move in #2")
  (check-false (valid-move? '((6 1) (2 13) (S V) (18 16) (21 17) (H S)) 4 5 '((6 1) (2 13) (S V) (18 16) (D V) (H S))) "Can't enter blocked door")
  (check-false (valid-move? test-state 0 6 '((O S) (2 13) (S V) (21 16) (21 17) (H S))) "Can't enter on suggestion")
  (check-true (valid-move? test-state 2 2 '((6 1) (2 13) (6 18) (21 16) (21 17) (H S))) "Basic move out")
  (check-true (valid-move? test-state 2 2 '((6 1) (2 13) (8 20) (21 16) (21 17) (H S))) "Move out onto other door")
  (check-false (valid-move? test-state 2 2 '((6 1) (2 13) (10 20) (21 16) (21 17) (H S))) "Move out too far")
  (check-true (valid-move? test-state 5 4 '((6 1) (2 13) (S V) (21 16) (21 17) (7 23))) "Multi door move out")
  (check-true (valid-move? test-state 5 2 '((6 1) (2 13) (S V) (21 16) (21 17) (10 17))) "Multi door move out #2")
  (check-true (valid-move? test-state 5 2 '((6 1) (2 13) (S V) (21 16) (21 17) (15 17))) "Multi door move out #3")
  (check-true (valid-move? test-state 5 4 '((6 1) (2 13) (S V) (21 16) (21 17) (4 20))) "Multi door move out over other door")
  (check-false (valid-move? test-state 5 2 '((6 1) (2 13) (S V) (21 16) (21 17) (17 18))) "Multi door move out too far")
  (check-false (valid-move? test-state 5 8 '((6 1) (2 13) (S V) (21 16) (21 17) (21 17))) "Multi door move out land on other player")
  (check-true (valid-move? test-state 2 0 '((6 1) (2 13) (K V) (21 16) (21 17) (H S))) "Secret passageway")
  (check-true (valid-move? test-room-state 0 0 '((U V) (C S) (B S) (K V) (O S) (U V))) "Secret passageway #2")
  (check-true (valid-move? test-room-state 1 0 '((C V) (U V) (B S) (K V) (O S) (U V))) "Secret passageway #3")
  (check-true (valid-move? test-room-state 3 0 '((C V) (C S) (B S) (S V) (O S) (U V))) "Secret passageway #4")
  (check-true (valid-move? test-room-state 5 0 '((C V) (C S) (B S) (K V) (O S) (C V))) "Secret passageway #5")
  (check-false (valid-move? test-room-state 0 0 '((O V) (C S) (B S) (K V) (O S) (C V))) "Not secret passageway")
  (check-true (valid-move? test-room-state 1 0 '((C V) (C V) (B S) (K V) (O S) (U V))) "Stay in room")
  (check-true (valid-move? test-room-state 2 0 '((C V) (C S) (B V) (K V) (O S) (U V))) "Stay in room #2")
  (check-false (valid-move? test-room-state 0 0 '((C S) (C S) (B S) (K V) (O S) (U V))) "Can't stay by suggestion in room")
  (check-true (valid-move? test-room-state 2 2 '((C V) (C S) (L V) (K V) (O S) (U V))) "Leave/enter multidoor room")
  (check-true (valid-move? test-room-state 2 7 '((C V) (C S) (L V) (K V) (O S) (U V))) "Leave/enter multidoor room #2")
  (check-true (valid-move? test-room-state 2 4 '((C V) (C S) (O V) (K V) (O S) (U V))) "Leave/enter multidoor room #3")
  (check-false (valid-move? test-room-state 2 2 '((C V) (C S) (L S) (K V) (O S) (U V))) "Can't switch rooms by suggestion")
  (check-false (valid-move? '((C V) (C S) (B S) (K V) (1 13) (U V)) 2 2 '((C V) (C S) (L V) (K V) (1 13) (U V))) "Can't leave through blocked door")
  (check-false (valid-move? test-state 1 2 '((6 3) (2 13) (S V) (21 16) (21 17) (H S))) "Wrong player moved")
  (check-false (valid-move? '((0 0) (24 17) (15 0) (10 0) (0 6) (0 19)) 0 2 '((0 2) (24 17) (15 0) (10 0) (0 6) (0 19))) "Bad coordinates")
  (check-false (valid-move? empty 0 2 test-state) "Poorly formed previous state")
  (check-false (valid-move? test-state 0 2 '(1 2 3 4 5 6)) "Poorly formed next state")
  (check-false (valid-move? test-state 0 2 '((26 1) (3 13) (S V) (21 16) (21 17) (H S))) "Poorly formed next state")
  (check-false (valid-move? test-state 6 0 test-state) "Invalid player num")
  (check-false (valid-move? test-state 1 1 '((6 1) (3 13) (S V) (21 16) (21 17) (H S))) "Invalid dice roll"))
